"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_utils_1 = require("web3-utils");
const melonjs_1 = require("@melonproject/melonjs");
const bignumber_js_1 = require("bignumber.js");
exports.address = accounting => accounting.contract.address;
exports.sharePrice = (accounting, _, context) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const calculations = yield context.loaders.accountingCalculations(accounting);
    return ((_a = calculations) === null || _a === void 0 ? void 0 : _a.sharePrice) ? web3_utils_1.fromWei(calculations.sharePrice.toFixed()) : new bignumber_js_1.BigNumber('NaN');
});
exports.grossAssetValue = (accounting, _, context) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    const calculations = yield context.loaders.accountingCalculations(accounting);
    return ((_b = calculations) === null || _b === void 0 ? void 0 : _b.gav) ? web3_utils_1.fromWei(calculations.gav.toFixed()) : new bignumber_js_1.BigNumber('NaN');
});
exports.netAssetValue = (accounting, _, context) => __awaiter(void 0, void 0, void 0, function* () {
    var _c;
    const calculations = yield context.loaders.accountingCalculations(accounting);
    return ((_c = calculations) === null || _c === void 0 ? void 0 : _c.nav) ? web3_utils_1.fromWei(calculations.nav.toFixed()) : new bignumber_js_1.BigNumber('NaN');
});
exports.holdings = (accounting, _, context) => __awaiter(void 0, void 0, void 0, function* () {
    const holdings = yield accounting.getFundHoldings(context.block);
    const output = context.environment.tokens.reduce((carry, current) => {
        var _a, _b;
        const holding = holdings.find(holding => melonjs_1.sameAddress(holding.address, current.address));
        if (!holding && current.historic) {
            return carry;
        }
        const add = {
            address: current.address,
            amount: (_b = (_a = holding) === null || _a === void 0 ? void 0 : _a.amount, (_b !== null && _b !== void 0 ? _b : new bignumber_js_1.BigNumber(0))),
        };
        return [...carry, add];
    }, []);
    return output.map(holding => [accounting, holding]);
});
